//INTERACTIVE VISUALIZATION OF PHOTONIC RUTHERFORD SCATTERING, Markus Selmke (selmke@rz.uni-leipzig.de, markus.selmke@gmx.de)

import controlP5.*;
ControlP5 controlP5;

/* @pjs preload="BG.jpg,Scan_RedGreen.png,Scan_Red.png,Scan_Red_PD.png,AuLens.png,AuLens_small.png,AuNP.png,AuNP_small.png,Zoom_Red.png,Zoom_Green.png,Zoom.png,PTSignalFormulaw.png,PTSignalFormulaPDw.png,PTSignalFormuladzfw.png,GreenLasers.png,RedLasers.png,Red_Beam_PT.png,Red_Beam_PTExpanded.png,Red_Beam_PTContracted.png,Signal0.png"; */
//image variables (i.e. .png formulas, particle images and background)
PImage imBG, imTr, imnr, imT0, imn0, imIdz, imIhz, imAuNP, imAuNPCap, imAuNPh, imAuLens, imGreenLaser, imRedLaser, imAuNPhNudge, imCH;
PImage im_DTK, im_w0d, im_R, im_Rnm, im_w0dnm, im_DrHBM, im_TrHBM, im_THBM, im_thetat, im_T0w, im_T0pDT, im_D0, im_null, im_p2pi, im_m2pi, im_mpi, im_ppi;
PImage img = createImage(1000,800,RGB);  // Make a PImage object
PImage im_DT, im_DHBMt, im_DTt, im_Phit, im_t;
String strParam0 = "";

//physics parameters
public float T0=25.0+273.0, DT=0.0, eta0=0.0, etainf=0.0000298, A=496.889, kB=1.3806505, theta=0.0, TV=152.0; 
public float n0=1.46, dndT=-3.6, DTMax=200.0, etaScale=100000, TScale=0.6, xp=400.0, zp=400.0, Angle=0.0, dzf=0.0, DTMaxF=400.0;
public float etaHBM=0.0, DHBM=0.0, THBM=0.0, eta0inf=0.0, etax=0.0, Tx=0.0, etarHBM=0.0, TrHBM=0.0, DrHBM=0.0, D0=0.0, Dr0=0.0;
public float MaxDrHBM=150, MaxDHBM=27000.0, MaxSignal=200;
public float lambdad=632.0/2.0, wh=220.0, whz=0.0, wz=3.6*220.0, wd=281.0, zRd=PI*wd*wd/lambdad, lambdah=532.0/2.0, zRh=PI*wh*wh/lambdah, Vgreen=100.0;
public float ta=0.0, Signal=0.0, Tempa=0.0, Signala=0.0, DHBMa=0.0, dti=0.001, S0=1, z=0.0, S=0.0;
public int Pred, RadiusP=20, Radius=20, RadiusOld=20, NudgeVelocity=10, NPAlpha, x, strX=10, TrajL=280, m=1, AngleLength=1055, TrajLs=1000;

//Photon Nudging Toggles and Variables
public float targetangle, acceptanceangle=20.0*PI/180.0, BFildStrength=0.2, BAngle=PI;

//other variables and plotting setup, i.e. standard width of Plots and sliders...
public float bdifz=0.0, bdifx=0.0, yPlot=0.0, xPlot=0.0, cosBA, sinBA;
public int bh=20, bw=200, bx=40, Tol=50, TrPlotExtend=100, ArrL=12, ArrW=7, PlotHight=75, kAct=0, Nparticles=1, Nparms=8, CHX=400, CHY=400, DoubleClick;
int k=0, j=0, i=0, kp=0;
public PFont font, CP5font;

//toggles for simulation and live-plotting, switched by user interface
public boolean HBMToggle, bover, locked, Show_NPPlot, ShowTrajectoryToggle, PlotTempsToggle, PlotDiffCoefsToggle, ShowParticleToggle;
public boolean BFieldToggle, PhotNudgeToggle, NudgeToggle, AutoNudgeToggle, PlotPhiToggle, PlotDTToggle, PlotAngleToggle, RadPressureToggle, ShowPTToggle;

public float[][][] Particles = new float[1000][9][1]; //[TrajLs][Nparms][Nparticles], array stores Particles and their trajectories
//[frame#][0][particle#]: x
//[1]: y
//[2]: DT
//[3]: Signal
//[4]: DHBM
//[5]: DrHBM
//[6]: THBM
//[7]: TrHBM
//[8]: Angle


void setup() { 
  size(1000, 800);
  smooth();
  frameRate(28);
  
  eta0 = etainf*exp(A/(T0-TV)); //viscosity at T0
  eta0inf = eta0/etainf; //viscosity ratio. used in HBM expressions
  kAct=0; //active particle is particle #1
  D0 = kB*T0/(6*PI*eta0*Radius); //translational diffusion constant at ambient temperature
  Dr0 = kB*T0/(8*PI*Radius*Radius*Radius*eta0); //rotational diffusion constant at ambient temperature
  
  //Max DHBMs, used for plotting...
    HBMparms(DTMax);
    MaxDrHBM = DrHBM;
    MaxDHBM = DHBM;
  
  //initial particle position
  for (k=0; k<Nparticles; k++) {
    for (int i = 0; i<TrajLs-1;i++) {
      Particles[i][0][k]=xp;
      Particles[i][1][k]=zp;
      Particles[i][4][k]=MaxDHBM;
      Particles[i][5][k]=MaxDrHBM;
      Particles[i][6][k]=THBM;
      Particles[i][7][k]=TrHBM;
    }
  }
  
  // The background image must be the same size as the parameters into the size() method.
  // is 800 x 800 pixels. Load all used images.
  imBG = loadImage("Background.jpg");
  imAuLens = loadImage("AuLens.png");
  imAuNP = loadImage("AuNP.png");
  imAuNPCap = loadImage("AuNPCap.png");
  imAuNPh = loadImage("AuNPh.png");
  imTr = loadImage("Tr.png");
  imnr = loadImage("nr.png");
  imn0 = loadImage("n0.png");
  imT0 = loadImage("T0.png");
  imIhz = loadImage("Ihz.png");
  imIdz = loadImage("Idz.png");
  imAuNPhNudge = loadImage("AuNPhNudge.png");
  imCH = loadImage("crosshaircyan.png");
  
  im_DTK = loadImage("formula_DTK.png");
  im_w0d = loadImage("formula_w0d.png");
  im_R = loadImage("formula_R.png");
  im_Rnm = loadImage("formula_Rnm.png");
  im_w0dnm = loadImage("formula_w0dnm.png");
  im_DT  = loadImage("formula_DT.png");
  im_DHBMt = loadImage("formula_DHBMt.png");
  im_DTt = loadImage("formula_DTt.png"); 
  im_Phit = loadImage("formula_Phit.png");
  im_t = loadImage("formula_t.png");
  im_T0w = loadImage("formula_T0w.png");
  im_T0pDT = loadImage("formula_T0pDT.png");
  im_D0 = loadImage("formula_D0.png");
  im_null = loadImage("formula_null.png");
  im_p2pi = loadImage("formula_p2pi.png");
  im_m2pi = loadImage("formula_m2pi.png");
  im_ppi = loadImage("formula_ppi.png");
  im_mpi = loadImage("formula_mpi.png");
  im_thetat = loadImage("formula_thetat.png");
  im_DrHBM = loadImage("formula_DrHBM.png");
  im_THBM = loadImage("formula_THBM.png");
  im_TrHBM = loadImage("formula_TrHBM.png");
  
  //imGreenLaser = loadImage("GreenLaser.png");
  //imRedLaser = loadImage("RedLaser.png");
  
  font = createFont("Arial", 12); 
  textFont(font);
  
  //initialize control sliders and checkboxes
  controlP5 = new ControlP5(this);
   ControlFont CP5font = new ControlFont(font); 
   controlP5.setControlFont(CP5font);
    
    //Slider redslider = controlP5.addSlider("red_laser_power", 0, 255, 0, 20, 550, bw, bh);
    Slider timestepslider = controlP5.addSlider("dtime", 0, 100, 0, 20, 675, bw, bh);
      timestepslider.setValue(1);
    Slider DTslider = controlP5.addSlider("DT", 0, 500, 0, 20, 725, bw, bh);
    Slider Radiusslider = controlP5.addSlider("RadiusSL", 0, 50, 0, 20, 625, bw, bh);
      Radiusslider.setValue(Radius);
    Slider whslider = controlP5.addSlider("beamwaist", 50.0, 800.0, 0, 20, 700, bw, bh);
    Slider dzfslider = controlP5.addSlider("focus_offset", -1, 1, 0, 20, 450, bh, 100);
    Slider Vgreenslider = controlP5.addSlider("greenVz", 0, 20.0, 0, 20, 650, bw, bh);
      Vgreenslider.setValue(2);
      
    Toggle HBM = controlP5.addToggle("HBM", HBMToggle, 20, 582, 20, 20);
      HBM.setValue(true);
    Toggle ShowProfiles = controlP5.addToggle("ShowProfiles", Show_NPPlot, 55, 582, 20, 20);
      ShowProfiles.setValue(true);
    Toggle ShowTrajectory = controlP5.addToggle("ShowTraj", ShowTrajectoryToggle, 90, 582, 20, 20);
      ShowTrajectory.setValue(true);
    Button ClearTrajButton = controlP5.addButton("ClearTrajectory",1.0,112,582,40,20);
    Toggle RadPressure = controlP5.addToggle("RadPressure", RadPressureToggle, 165, 582, 20, 20);
      RadPressure.setValue(false);
    Toggle ShowPart = controlP5.addToggle("ShowParticle", ShowParticleToggle, 250, 582, 20, 20);
      ShowPart.setValue(true);
      
    Toggle ShowPT = controlP5.addToggle("PTSignalDisplay", ShowPTToggle, 890, 717, 20, 20);
      ShowPT.setValue(false);
    Toggle Nudge = controlP5.addToggle("PhotonNudging", PhotNudgeToggle, 800, 717, 20, 20);
      Nudge.setValue(false);
    Toggle AutoNudge = controlP5.addToggle("Target", AutoNudgeToggle, 750, 717, 20, 20);
      AutoNudge.setValue(false);
    Slider NudgeVelocitySlider = controlP5.addSlider("NudgeVelocitySL", 0, 100, 0, 750, 690, bw-40, bh);
      NudgeVelocitySlider.setValue(NudgeVelocity);
    Slider AcceptanceAngleSlider = controlP5.addSlider("AceptanceAngleSL", 0, 90, 0, 750, 665, bw-40, bh);
      AcceptanceAngleSlider.setValue(20);
    
    Toggle BField = controlP5.addToggle("BField", BFieldToggle, 750, 623, 20, 20);
      BField.setValue(false); 
    Slider BStrengthnslider = controlP5.addSlider("BStrength", 0, 1.0, 0, 750, 595, bw-40, bh);
    Slider BAngleslider = controlP5.addSlider("BAngleSL", -360, 360, 0, 750, 570, bw-40, bh);
    
    Toggle PlotPhi = controlP5.addToggle("PlotPhi", PlotPhiToggle, 810, 5, 20, 20);
      PlotPhi.setValue(true);
    Toggle PlotTs = controlP5.addToggle("PlotTemperatures", PlotTempsToggle, 870, 5, 20, 20);
      PlotTs.setValue(true);
    Toggle PlotDs = controlP5.addToggle("PlotDiffcoeffs", PlotDiffCoefsToggle, 930, 5, 20, 20);
      PlotDs.setValue(true);
    Toggle PlotDT = controlP5.addToggle("PlotDT", PlotDTToggle, 750, 5, 20, 20);
      PlotDT.setValue(true);
    Toggle PlotAngle = controlP5.addToggle("Plotangle", PlotAngleToggle, 690, 5, 20, 20);
      PlotAngle.setValue(true);
      
    
    //change their apperance!
    PlotTs.captionLabel().toUpperCase(false);
    PlotDs.captionLabel().toUpperCase(false);
    PlotPhi.captionLabel().toUpperCase(false);
    DTslider.captionLabel().toUpperCase(false);
    Radiusslider.captionLabel().toUpperCase(false);
    whslider.captionLabel().toUpperCase(false);
    HBM.captionLabel().toUpperCase(false);
    dzfslider.captionLabel().toUpperCase(false);
    ShowProfiles.captionLabel().toUpperCase(false);
    timestepslider.captionLabel().toUpperCase(false);
    ShowTrajectory.captionLabel().toUpperCase(false);
    PlotDT.captionLabel().toUpperCase(false);
    PlotAngle.captionLabel().toUpperCase(false);
    RadPressure.captionLabel().toUpperCase(false);
    Vgreenslider.captionLabel().toUpperCase(false);
    ShowPT.captionLabel().toUpperCase(false);
    ClearTrajButton.captionLabel().toUpperCase(false);
    Nudge.captionLabel().toUpperCase(false);
    AutoNudge.captionLabel().toUpperCase(false);
    NudgeVelocitySlider.captionLabel().toUpperCase(false);
    AcceptanceAngleSlider.captionLabel().toUpperCase(false);
    BField.captionLabel().toUpperCase(false);
    BStrengthnslider.captionLabel().toUpperCase(false);
    BAngleslider.captionLabel().toUpperCase(false);
    ShowPart.captionLabel().toUpperCase(false);
    
    PlotPhi.setCaptionLabel("Phi(t)");
    PlotTs.setCaptionLabel("T_HBM(t)");
    PlotDs.setCaptionLabel("D_HBM(t)");
    PlotDT.setCaptionLabel("DT(t)");
    PlotAngle.setCaptionLabel("Angle(t)");
    ShowPart.setCaptionLabel("Particle");
    Nudge.setCaptionLabel("Phot. Nudging");
    AutoNudge.setCaptionLabel("Target");
    NudgeVelocitySlider.setCaptionLabel("Nudge Velocity");
    AcceptanceAngleSlider.setCaptionLabel("Accept. Angle");
    BField.setCaptionLabel("B-Field");
    BStrengthnslider.setCaptionLabel("Field Strength");
    BAngleslider.setCaptionLabel("Field Direction");
    
    ClearTrajButton.setCaptionLabel("Clear");
    ShowPT.setCaptionLabel("Show PT Det. Vol.");
    Vgreenslider.setCaptionLabel("Radiation Induced Flow");
    RadPressure.setCaptionLabel("Rad. Pressure");
    whslider.setCaptionLabel("Heating Beam Waist");
    ShowTrajectory.setCaptionLabel("Trajectory");
    timestepslider.setCaptionLabel("Time-Step");
    ShowProfiles.setCaptionLabel("T(r)");
    dzfslider.setCaptionLabel("Laser Offset");
    DTslider.setCaptionLabel("Max Temp Elevation");
    Radiusslider.setCaptionLabel("Particle Radius");
    
      DTslider.setValue(DTMax);
      whslider.setValue(wh);
      BStrengthnslider.setValue(0.2);
      BAngleslider.setValue(0);
      
    UpdateBG();
}

void PhotonNudging(boolean theFlag) {
  PhotNudgeToggle=theFlag;
  
  controlP5.controller("PTSignalDisplay").setValue(0);
  controlP5.controller("ShowProfiles").setValue(0);
  controlP5.controller("PlotPhi").setValue(0);
  controlP5.controller("PTSignalDisplay").setValue(0);
  controlP5.controller("RadPressure").setValue(0);
  
  if (PhotNudgeToggle==true) {
    controlP5.controller("PlotDiffcoeffs").setValue(0);
    controlP5.controller("PlotTemperatures").setValue(0);
    controlP5.controller("HBM").setValue(0);
  } else {
    NudgeToggle = false; //turn off in case it was in on-state while switching
    controlP5.controller("PlotDiffcoeffs").setValue(1);
    controlP5.controller("PlotTemperatures").setValue(1);
    controlP5.controller("HBM").setValue(1);
  }
  UpdateBG();
}

void Target(boolean theFlag) {
  AutoNudgeToggle=theFlag;
}

void dtime(float theValue) {
  dti=theValue*0.001;
}

void greenVz(float theValue) {
  Vgreen=theValue;
}

void BField(boolean theFlag) {
  BFieldToggle=theFlag;
}

void BStrength(float theValue) {
  BFildStrength=theValue;
}

void BAngleSL(float theValue) {
  BAngle=theValue*PI/180.0;
}

void beamwaist(int theValue) {
  wh=theValue/2;
  zRh=PI*wh*wh/lambdah;
  UpdateBG();
}

void NudgeVelocitySL(int theValue) {
  NudgeVelocity=theValue;
}

void AceptanceAngleSL(int theValue) {
  acceptanceangle=theValue*PI/180.0;
}

void RadiusSL(int theValue) {
  //set new radius, store old for rescaling
    RadiusOld=Radius;
    Radius=theValue;
  //recalculate the ambient temperature diffusion coefficients
    D0 = kB*T0/(6*PI*eta0*Radius);
    Dr0 = kB*T0/(8*PI*Radius*Radius*Radius*eta0);
  //rescale the maximum HBM diffusion constants
    MaxDHBM = MaxDHBM*RadiusOld/Radius;
    MaxDrHBM = MaxDrHBM*RadiusOld*RadiusOld*RadiusOld/(Radius*Radius*Radius);
}

void ClearTrajectory(int theValue) {
  //clear Trajectory
  for (k=0; k<Nparticles; k++) {
    xp = Particles[0][0][k];
    zp = Particles[0][1][k];
    for (i = 1; i<TrajLs-1;i++) {
      Particles[i][0][k]=xp;
      Particles[i][1][k]=zp;
    }
  }
}

void DT(float theValue) {
  DTMax=theValue;
  UpdateBG();
}

void focus_offset(float theValue) {
  dzf=theValue;
  UpdateBG();
}

void wdslider(float theValue) {
  wd=theValue;
  UpdateBG();
}

void HBM(boolean theFlag) {
  HBMToggle=theFlag;
}

void ShowTraj(boolean theFlag) {
  ShowTrajectoryToggle=theFlag;
}

void ShowProfiles(boolean theFlag) {
  Show_NPPlot=theFlag;
}

void PlotPhi(boolean theFlag) {
  PlotPhiToggle=theFlag;
}

void PlotTemperatures(boolean theFlag) {
  PlotTempsToggle=theFlag;
}

void PlotDiffcoeffs(boolean theFlag) {
  PlotDiffCoefsToggle=theFlag;
}

void PlotDT(boolean theFlag) {
  PlotDTToggle=theFlag;
}

void ShowParticle(boolean theFlag) {
  ShowParticleToggle=theFlag;
}

void Plotangle(boolean theFlag) {
  PlotAngleToggle=theFlag;
}

void RadPressure(boolean theFlag) {
  RadPressureToggle=theFlag;
}

void PTSignalDisplay(boolean theFlag) {
  ShowPTToggle=theFlag;
}

void draw() 
{
  //set background image, load font, write title
    background(0);
    if (PhotNudgeToggle==true) {
      image(imBG,0,0);
    } else {
      image(img,0,0);
    }
    //if nudging is active, show some green color to indicate that the widefield illumination is on
    if (NudgeToggle==true) {
      fill(0,255,0,30); noStroke();
      //quad(-1,-1,-1,801,1001,801,1001,-1); //whole screen
      quad(740,557,740,795,995,795,995,557); //rectangle around interface area
    }
    
    //draw grid of white lines to indicate B-field direction
    if (BFieldToggle==true && PhotNudgeToggle==true) {
      strokeWeight(2); stroke(255,30);
      cosBA = cos(BAngle); sinBA = sin(BAngle);
      for (k=-25; k<25; k++) {
        line(-cosBA*1500-sinBA*k*50,-sinBA*1500+cosBA*k*50,cosBA*1500-sinBA*k*50,sinBA*1500+cosBA*k*50);
      }
    }
    
    //print some text: Frame caption etc.
    fill(255);
    if (AutoNudgeToggle==true && PhotNudgeToggle==true) {
      text("double-click to set target!", 15, 12);
    } else {
      text("click and drag AuNP!", 15, 12);
    }
    frame.setTitle("HOT BROWNIAN MOTION THROUGH A HEATING FOCUS");
    
    if (PhotNudgeToggle==false) {
      //plot the heating laser gaussian equi-intensity lines!
         //plot beam waist
         strokeWeight(2); strokeJoin(ROUND); stroke(0,255,0,50); noFill();
         beginShape(); 
           for (int i=0; i<800;i=i+5) {
              whz = wh*sqrt((i-(400+dzf*zRh))*(i-(400+dzf*zRh))/(zRh*zRh)+1);
              vertex(400+whz,i);
           }
         endShape();
         beginShape();
           for (int i=0; i<800;i=i+5) {
              whz = wh*sqrt((i-(400+dzf*zRh))*(i-(400+dzf*zRh))/(zRh*zRh)+1);
              vertex(400-whz,i);
           }
         endShape();
    
      //vertical optical axis
        strokeWeight(2); stroke(255,100); 
        line(400,0,400,800); 
        //annotate optical axis (vertical)
        textAlign(CENTER, CENTER); fill(255); 
        rotate(PI/2); text("optical axis", 55, -415); rotate(-PI/2);
        textAlign(LEFT, BASELINE) ;
    }
      
    //PLOT PT SIGNAL SHAPE
    if (ShowPTToggle==true && PhotNudgeToggle==false) {
      S0=1; wh=wh/3.0; dzf=dzf*2.0; wz=wh*3.6;
      strokeWeight(2); strokeJoin(ROUND);
      for (S=0.1; S<=0.9; S=S+0.6) {
        //positive lobe
        fill(0,0,255,50);
        beginShape();
        for (z=-400; z<400; z=z+2) {
          if (log((z+dzf*wh)*S0/S) > 2*z*z/(wz*wz)) {
            vertex(400+(wh/sqrt(2))*sqrt(-2*z*z/(wz*wz)+log((z+dzf*wh)*S0/S)),400+z);
          }
         }
         for (z=400; z>-400; z=z-2) {
           if (log((z+dzf*wh)*S0/S) > 2*z*z/(wz*wz)) {
             vertex(400-(wh/sqrt(2))*sqrt(-2*z*z/(wz*wz)+log((z+dzf*wh)*S0/S)),400+z);
           }
         }
        endShape();
        //negative lobe
        fill(255,0,0,50);
        beginShape();
        for (z=-400; z<400; z=z+2) {
          if (log(-(z+dzf*wh)*S0/S) > 2*z*z/(wz*wz)) {
            vertex(400+(wh/sqrt(2))*sqrt(-2*z*z/(wz*wz)+log(-(z+dzf*wh)*S0/S)),400+z);
          }
         }
         for (z=400; z>-400; z=z-2) {
           if (log(-(z+dzf*wh)*S0/S) > 2*z*z/(wz*wz)) {
             vertex(400-(wh/sqrt(2))*sqrt(-2*z*z/(wz*wz)+log(-(z+dzf*wh)*S0/S)),400+z);
           }
         }
         endShape();
      }
      wh=3.0*wh; dzf=dzf/2.0;
    }
      
    //get current position and corresponding Temperature Increment & rel. PT Signal! Nparticles
    m=m+1;
    for (k=0; k<Nparticles; k++) { 
      xp = Particles[0][0][k];
      zp = Particles[0][1][k];
      
      //get the HBM parameters, Signal and DT
      getHBMparms();
      
      Particles[0][2][k] = DT;
      Particles[0][3][k] = Signal;
      Particles[0][4][k] = DHBM;
      Particles[0][5][k] = DrHBM;
      Particles[0][6][k] = THBM;
      Particles[0][7][k] = TrHBM;
      Particles[0][8][k] = Angle;
      
      DoDiffusionStep();
      
      Particles[0][0][k] = xp;
      Particles[0][1][k] = zp;
      
      ShiftTraces();
      
      //Plot Particle Trajectory
      if (ShowTrajectoryToggle==true) {
        noFill(); strokeWeight(2); strokeCap(ROUND);
        //for (int i=0; i<TrajLs-2;i++) {
        for (int i=TrajLs-3; i>=0;i--) {
          ta = (TrajLs-i*1.0)/TrajLs; //linear transparency based on time
          //ta = (i*1.0)/TrajLs; //linear transparency based on time
          //color (red,blue) based on: distance to center or: temperature
          if (PhotNudgeToggle==true && AutoNudgeToggle==true) {
            Tempa = 1.0 - sqrt((Particles[i][0][k]-CHX)*(Particles[i][0][k]-CHX) + (Particles[i][1][k]-CHY)*(Particles[i][1][k]-CHY)) / 100.0;
          } else {
            Tempa = Particles[i][2][k]/DTMax;
          }
          stroke(255.0*Tempa,255*(1.0-Tempa),255*(1.0-Tempa),255.0*ta);
          line(Particles[i][0][k],Particles[i][1][k],Particles[i+1][0][k],Particles[i+1][1][k]);
        }
      }
      
      //draw cross-hair and acceptance angle if in nudging mode and target automatic mode
      if (AutoNudgeToggle==true && PhotNudgeToggle==true) {
         tint(255, 100);
         image(imCH,CHX-24,CHY-24);
         //write acceptance angle  
         fill(255); text("target angle: "+str(targetangle*180.0/PI),20,90);
      }
      
      if (ShowParticleToggle==true) {
        //draw Particle image (cold & hot on top), also add axis fixed through particle
        pushMatrix();
          tint(255,255);
          translate(xp,zp);
          rotate(Angle);
          stroke(255,255); strokeWeight(2); 
          line(-1.2*RadiusP,0,1.2*RadiusP,0);
          if (PhotNudgeToggle == true) {
            if (NudgeToggle==true) {
              image(imAuNPhNudge, -210, -198); //draws hot capped particle
            } else {
              image(imAuNPCap, -210, -198); //draws cold capped particle
            } 
          } else {
            image(imAuNP, -210, -198); //cold AuNP
            tint(255, 255.0*DT/DTMax);
            image(imAuNPh, -210, -198); //hot AuNP
          }
          //Additional Drawing on Particle for Nudging
         if (PhotNudgeToggle == true) { 
           if (AutoNudgeToggle == true) {
              //automiatic mode: draw acceptance angle
              if (NudgeToggle == true) {
                fill(255,0,0,80);
              } else {
                fill(255,0,0,20);
              }
              stroke(255,0,0,70); strokeWeight(2);
              triangle(AngleLength*RadiusP*sin(acceptanceangle),-AngleLength*RadiusP*cos(acceptanceangle),-AngleLength*RadiusP*sin(acceptanceangle),-AngleLength*RadiusP*cos(acceptanceangle),0,0);
           } else {
            //for manual nudging: draw arrow for nudge direction indicator
            stroke(255,0,0,50); strokeWeight(3); fill(255,0,0,50); 
            triangle(-10,-RadiusP,10,-RadiusP,0,-1.4*RadiusP);
           } 
         }
        popMatrix();
      }
      tint(255,255);
      //end per-particle actions
    }
    
      
      //Plot DT(t)
      yPlot = 70;
      xPlot = 980;
      if (PlotDTToggle==true) {
        image(im_DTt,xPlot+10-TrajL,yPlot-10);
        strokeWeight(2); stroke(255,255); fill(255);
          line(xPlot-TrajL,yPlot,xPlot-TrajL,yPlot+PlotHight); //vertical axis
          line(xPlot-TrajL,PlotHight+yPlot,xPlot,PlotHight+yPlot); //horizontal axis
          triangle(xPlot-TrajL,yPlot-ArrL,xPlot-TrajL+ArrW/2,yPlot,xPlot-TrajL-ArrW/2,yPlot);
        strokeWeight(2); strokeJoin(ROUND); stroke(255,0,0,255); noFill();
          beginShape();
          vertex(xPlot-TrajL-5,PlotHight+yPlot-PlotHight*Particles[0][2][kAct]/DTMax);
          for (int i=0; i<TrajL-2;i++) {
            Tempa = PlotHight*Particles[i][2][kAct]/DTMax;
            vertex(xPlot-TrajL+i,PlotHight+yPlot-Tempa);
          }
          endShape();
          yPlot=yPlot+PlotHight+20;
      }
      
      //Plot THBM(t)
      if (PlotTempsToggle==true) {
        image(im_THBM,xPlot+10-TrajL,yPlot-10); text("/",xPlot-TrajL+60,yPlot+3);
        image(im_TrHBM,xPlot-TrajL+70,yPlot-10);
        image(im_T0pDT,xPlot-TrajL-62,yPlot);
        image(im_T0w,xPlot-TrajL-25,yPlot+PlotHight-10);
        strokeWeight(2); stroke(255,255); fill(255);
          line(xPlot-TrajL-5,yPlot+5,xPlot-TrajL,yPlot+5); //T0+DT line
          line(xPlot-TrajL,yPlot,xPlot-TrajL,yPlot+PlotHight); //vertical axis
          line(xPlot-TrajL-5,PlotHight+yPlot,xPlot,PlotHight+yPlot); //horizontal axis
          triangle(xPlot-TrajL,yPlot-ArrL,xPlot-TrajL+ArrW/2,yPlot,xPlot-TrajL-ArrW/2,yPlot);
        strokeWeight(2); strokeJoin(ROUND); stroke(255,0,0,255); noFill();
          line(xPlot-TrajL+10,yPlot-12,xPlot-TrajL+22,yPlot-12);
          beginShape(); 
          vertex(xPlot-TrajL-5,PlotHight+yPlot-(PlotHight-5)*(Particles[0][6][kAct]-T0)/DTMax);
          for (int i=0; i<TrajL-2;i++) {
            Tempa = (PlotHight-5)*(Particles[i][6][kAct]-T0)/DTMax;
            vertex(xPlot-TrajL+i,PlotHight+yPlot-Tempa);
          }
          endShape();
        //rotational THBM  
        strokeWeight(2); strokeJoin(ROUND); stroke(240,165,0,255); noFill();
          line(xPlot-TrajL+70,yPlot-12,xPlot-TrajL+86,yPlot-12);
          beginShape(); 
          vertex(xPlot-TrajL-5,PlotHight+yPlot-(PlotHight-5)*(Particles[0][7][kAct]-T0)/DTMax);
          for (int i=0; i<TrajL-2;i++) {
            Tempa = (PlotHight-5)*(Particles[i][7][kAct]-T0)/DTMax;
            vertex(xPlot-TrajL+i,PlotHight+yPlot-Tempa);
          }
          endShape();
          yPlot=yPlot+PlotHight+20;
      }
      
      //Plot Signal(t)
      if (PlotPhiToggle==true) {
        image(im_Phit,xPlot-TrajL+10,yPlot-10);
        strokeWeight(2); stroke(255,255); fill(255);
          line(xPlot-TrajL,yPlot,xPlot-TrajL,yPlot + PlotHight); //vertical axis
          line(xPlot-TrajL,yPlot + PlotHight,xPlot,yPlot + PlotHight); //horizontal axis
          triangle(xPlot-TrajL,yPlot-ArrL,xPlot-TrajL+ArrW/2,yPlot,xPlot-TrajL-ArrW/2,yPlot);
        //translational THBM
        strokeWeight(2); strokeJoin(ROUND); stroke(0,140,250,100); noFill();
          line(xPlot-TrajL,yPlot + 0.5*PlotHight,xPlot,yPlot + 0.5*PlotHight);
          stroke(0,140,250,255);
          beginShape();
          vertex(xPlot-TrajL-5,yPlot + 0.5*PlotHight-PlotHight*Particles[0][3][kAct]/MaxSignal);
          for (int i=0; i<TrajL-2;i++) {
            Signala = PlotHight*Particles[i][3][kAct]/MaxSignal;
            vertex(xPlot-TrajL+i,yPlot + 0.5*PlotHight - Signala);
          }
          endShape();
          yPlot=yPlot+PlotHight+20;
      }
      
      //Plot DHBM(t)
      if (PlotDiffCoefsToggle==true) {
        image(im_DHBMt,xPlot-TrajL+10,yPlot-10); text("/",xPlot-TrajL+90,yPlot+3);
        image(im_DrHBM,xPlot-TrajL+100,yPlot-10);
        image(im_D0,xPlot-TrajL-52,yPlot+PlotHight-10);
        strokeWeight(2); stroke(255,255); fill(255);
          line(xPlot-TrajL,yPlot,xPlot-TrajL,yPlot + PlotHight); //vertical axis
          line(xPlot-TrajL-5,yPlot + PlotHight,xPlot,yPlot + PlotHight); //horizontal axis
          triangle(xPlot-TrajL,yPlot-ArrL,xPlot-TrajL+ArrW/2,yPlot,xPlot-TrajL-ArrW/2,yPlot);
        strokeWeight(2); strokeJoin(ROUND); stroke(155,140,250,255); noFill();
          line(xPlot-TrajL+12,yPlot-12,xPlot-TrajL+24,yPlot-12);
          beginShape();
          vertex(xPlot-TrajL-5,yPlot + PlotHight - (PlotHight-10)*(Particles[0][4][kAct]-D0)/(MaxDHBM-D0));
          for (int i=0; i<TrajL-2;i++) {
            DHBMa = (PlotHight-10)*(Particles[i][4][kAct]-D0)/(MaxDHBM-D0);
            vertex(xPlot-TrajL+i,yPlot + PlotHight-DHBMa);
          }
          endShape();
        //rotational DHBM  
        strokeWeight(2); strokeJoin(ROUND); stroke(238,130,238,255); noFill();
          line(xPlot-TrajL+102,yPlot-12,xPlot-TrajL+124,yPlot-12);
          beginShape();
          vertex(xPlot-TrajL-5,yPlot + PlotHight -(PlotHight-10)*(Particles[0][5][kAct]-Dr0)/(MaxDrHBM-Dr0));
          for (int i=0; i<TrajL-2;i++) {
            DHBMa = (PlotHight-10)*(Particles[i][5][kAct]-Dr0)/(MaxDrHBM-Dr0);
            vertex(xPlot-TrajL+i,yPlot + PlotHight-DHBMa);
          }
          endShape();
          yPlot=yPlot+PlotHight+20;
      }
      
    //Plot Angle(t)
      if (PlotAngleToggle==true) {
        image(im_thetat,xPlot-TrajL+10,yPlot-10);
        image(im_null,xPlot-TrajL-13,yPlot+PlotHight/2-6);
        image(im_m2pi,xPlot-TrajL-30,yPlot+PlotHight-6);
        image(im_p2pi,xPlot-TrajL-30,yPlot-6);
        strokeWeight(2); stroke(255,255); fill(255);
          line(xPlot-TrajL,yPlot,xPlot-TrajL,yPlot + PlotHight); //vertical axis
          line(xPlot-TrajL,yPlot + PlotHight,xPlot,yPlot + PlotHight); //horizontal axis
          triangle(xPlot-TrajL,yPlot-ArrL,xPlot-TrajL+ArrW/2,yPlot,xPlot-TrajL-ArrW/2,yPlot);
        strokeWeight(2); strokeJoin(ROUND); stroke(255,100); noFill();
          line(xPlot-TrajL,yPlot + 0.5*PlotHight,xPlot,yPlot + 0.5*PlotHight);
        stroke(255,255);
          beginShape();
          vertex(xPlot-TrajL-5,yPlot + PlotHight*(0.5-Particles[0][8][kAct]/(4*PI)));
          for (int i=0; i<TrajL-2;i++) {
            Signala = PlotHight*Particles[i][8][kAct]/(4*PI);
            vertex(xPlot-TrajL+i,yPlot + 0.5*PlotHight - Signala);
          }
          endShape();
      }
  
  xp = Particles[0][0][kAct];
  zp = Particles[0][1][kAct];
  getHBMparms();
  
  //baseline for T(r) and n(r)  
    if (Show_NPPlot==true) {
      strokeWeight(2); stroke(255); 
      line(xp-TrPlotExtend,zp,xp+TrPlotExtend,zp); //horizontal axis
      line(xp,zp+TrPlotExtend,xp,zp-TrPlotExtend); //vertical axis
      triangle(xp,zp-TrPlotExtend-ArrL,xp+ArrW/2,zp-TrPlotExtend,xp-ArrW/2,zp-TrPlotExtend);
      triangle(xp+TrPlotExtend+ArrL,zp,xp+TrPlotExtend,zp+ArrW/2,xp+TrPlotExtend,zp-ArrW/2);
    }
    
    //PLOT T(r) & n(r) Profile around diffusing particle
    noFill();
    if (Show_NPPlot==true) {
      tint(255, 255);
      noFill(); strokeWeight(2); strokeJoin(ROUND); 
      stroke(200,0,0); 
      beginShape();
      for (x=-TrPlotExtend; x<=TrPlotExtend; x=x+2) {
        if (abs(x) > RadiusP) {
          Tx = DT*RadiusP/abs(x);
          vertex(xp+x,zp - TScale*Tx);
        } else {
          vertex(xp+x,zp - TScale*DT);
        }
      }  
      endShape();
      image(imTr, xp+45, zp - 20 - TScale*DT*RadiusP/50); //png: T(r)
      //viscosity Profile
      stroke(0,200,200);
      beginShape();
      for (x=-TrPlotExtend; x<=TrPlotExtend; x=x+2) {
        if (abs(x) > RadiusP) {
          etax = etainf*exp(A/(T0 + DT*RadiusP/abs(x) - TV));
          vertex(xp+x,zp + TrPlotExtend - etaScale*etax);
        }
      }  
      endShape();
      //display formulas: T_0, n_0, n(r)
      image(imnr, xp+41, zp + TrPlotExtend - etaScale*etainf*exp(A/(T0 + DT*RadiusP/50 - TV))); //png: T(r)
      image(imn0, xp-TrPlotExtend, zp + TrPlotExtend - etaScale*eta0);
      image(imT0, xp-TrPlotExtend, zp - 20);
    }
    
    //print some text, e.g. current diffusion coeff. etc.
      fill(255);
      //text("HBM Diffusion coeff.: "+str(DHBM),20,30);
      text("Particle Temperature: "+str(DT),20,30);
      text("THBM-T0: "+str(THBM-T0),20,45);
      text("TrHBM-T0: "+str(TrHBM-T0),20,60);
      text("angle: "+str(Angle*180.0/PI),20,75);
      
      //text("norm. Particle Temperature: "+str(Traj_THBM[0]/(T0+DTMax)),20,130);
      //text("norm. Signal: "+str(Traj_Signal[0]/MaxSignal),20,145);
      //text("norm. DHBM: "+str(Traj_DHBM[0]/MaxDHBM),20,160);
      //text("norm. rot DHBM: "+str(Traj_DrHBM[0]/MaxDrHBM),20,175);
    
    //needed for particle dragging
    if (mouseX > xp-2*RadiusP && mouseX < xp+2*RadiusP && mouseY > zp-2*RadiusP && mouseY < zp+2*RadiusP) {
      bover = true;  
    } else {
      bover = false;
    }
    
    if (DoubleClick > 0) {
      DoubleClick = DoubleClick - 1;
    }
}

//Toggle Zoom-Image
void mousePressed() {
  //Toggle Locked state
  if(bover) { 
    locked = true; 
  } else {
    locked = false;
  }
  bdifx = mouseX - xp; 
  bdifz = mouseY - zp;
  
  if (DoubleClick > 0) {
    CHX = mouseX;
    CHY = mouseY;
  }
  DoubleClick = 5;
}

void mouseDragged() {
  if(locked) {
    xp = mouseX-bdifx;
    zp = mouseY-bdifz; 
    Particles[0][0][kAct] = xp;
    Particles[0][1][kAct] = zp;
  }
}

void mouseReleased() {
  locked = false;
}

// return a random value in the range [-1..1] with normal distribution around 0.
// Implements the Marsaglia Polar Method, as described in wikipedia, but only returns one of the values.
float randomNormal()
{
  float x = 1.0, y = 1.0, 
        s = 2.0; // s = x^2 + y^2
  while(s >= 1.0)
  {
    x = random(-1.0f, 1.0f);
    y = random(-1.0f, 1.0f);
    s = x*x + y*y;
  }
  return x * sqrt(-2.0f * log(s)/s);
}

float signum(float f) {
  if (f > 0) return 1.0;
  if (f < 0) return -1.0;
  return 0;
} 

void UpdateBG() {
  img.loadPixels();
  imBG.loadPixels();
  if (PhotNudgeToggle==false) {
    for (int x = 0; x < width; x++) {
      for (int y = 0; y < height; y++) {
        int loc = x + y * width;
        float r = red   (imBG.pixels[loc]);
        float g = green (imBG.pixels[loc]);
        float b = blue  (imBG.pixels[loc]);
        
        g = g + (DTMax/DTMaxF)*220*exp(-2*(400-x)*(400-x)/(wh*wh))/((y-(400+dzf*zRh))*(y-(400+dzf*zRh))/(zRh*zRh) + 1.0);
        img.pixels[loc] = color(r,g,b);
      }
    }
  }
  img.updatePixels();
}

void getHBMparms() {
  //get DT and Signal for current position
  if (PhotNudgeToggle == false) {
    DT = DTMax*exp(-2*(xp-400.0)*(xp-400.0)/(wh*wh))/((zp-(400+dzf*zRh))*(zp-(400+dzf*zRh))/(zRh*zRh) + 1.0);
    Signal = (DT/DTMax)*(zp-400)*exp(-2*(xp-400.0)*(xp-400.0)/(wd*wd))/((zp-400)*(zp-400)/(zRh*zRh) + 1.0);
      Signal = Signal + 0.005*randomNormal()*MaxSignal; //add some noise
  } else {
    if (NudgeToggle == true) {
       DT = DTMax*1.0;
    } else {
       DT = 0.001;
    } 
  }
  //Diffision coefficient based on current particle temperature
    if (HBMToggle==true) {
      HBMparms(DT);
    } else {
      THBM = T0;
      DHBM = D0; //no HBM, Diffusion at T=T0
      TrHBM = T0;
      DrHBM = Dr0;
    }  
}

//HBM Theory
void HBMparms(float DTp) {
  theta = DTp/(T0-TV);
  //translational HBM
    THBM = T0*(1.0 + DTp/(2*T0)+(log(eta0inf)-1)*DTp*DTp/(12.0*T0*T0));
    etaHBM = eta0/(1.0 + (193.0/486)*log(eta0inf)*theta - ((56.0/243)*log(eta0inf)-(12563.0/118098)*log(eta0inf)*log(eta0inf))*theta*theta);
    DHBM = kB*THBM/(6*PI*etaHBM*Radius);
  //rotational HBM
    etarHBM = eta0/(1 + (3.0/4.0)*A*theta*theta/DTp - (3.0/10.0)*A*theta*theta*theta*theta*(2*T0-2*TV-A)/(DTp*DTp));
    TrHBM = T0 + (3.0/4.0)*DTp + (3.0/80.0)*A*theta*theta;
    DrHBM = kB*TrHBM/(8*PI*Radius*Radius*Radius*etarHBM);
}

void DoDiffusionStep() {
  //angular diffusion
    Angle = Angle + 0.05*DrHBM*randomNormal(); //rotation angle
    Angle = Angle % (2*PI);
    if (BFieldToggle==true && PhotNudgeToggle==true) {
      Angle = Angle - (Angle-BAngle)*BFildStrength; //orientational torque acting as restoring spring
    }
  //determine angle to target, check wether orientation points toward acceptance angle
  if (PhotNudgeToggle==true && AutoNudgeToggle==true) {
    targetangle = atan2(xp-CHX,zp-CHY) + Angle;
    if ((abs(targetangle) < acceptanceangle) || ((abs(targetangle) > (2*PI-acceptanceangle)) && (abs(targetangle) < (2*PI+acceptanceangle)))) {
      NudgeToggle=true;
    } else {
      NudgeToggle=false;
    }
  }
  //move particle
    if (locked==false) {
      //move Particle by Diffusion
        xp = xp + randomNormal()*DHBM*dti;
        zp = zp + randomNormal()*DHBM*dti;
      //move particle through flow
        if (RadPressureToggle==true && PhotNudgeToggle==false) {
          zp = zp - Vgreen*exp(-2*(xp-400.0)*(xp-400.0)/(wh*wh))/((zp-(400+dzf*zRh))*(zp-(400+dzf*zRh))/(zRh*zRh) + 1.0);
          xp = xp - 0.02*(xp-400)*Vgreen*exp(-2*(xp-400.0)*(xp-400.0)/(wh*wh/2))/((zp-(400+dzf*zRh))*(zp-(400+dzf*zRh))/(zRh*zRh) + 1.0);
        }
      //nudge
        if (NudgeToggle==true && PhotNudgeToggle==true) {
          xp = xp + sin(Angle)*NudgeVelocity;
          zp = zp - cos(Angle)*NudgeVelocity;
        }  
    }
  //reflecting boundary conditions
    if (xp > 1000) {
     xp = xp - (xp-1000);
    }
    if (xp < 0) {
     xp = -xp;
    }
    if (zp > 750) {
     zp = zp - (zp-750);
    }
    if (zp < 0) {
     zp = -zp;
    }    
}

void ShiftTraces() {
  for (kp=0; kp<Nparticles; kp++) {
    for (j=0; j<=Nparms; j++) {
      for (i=TrajLs-2; i>=0; i=i-1) {
        Particles[i+1][j][kp] = Particles[i][j][kp];
      }
    }
  }
}

//change active Particle
void keyPressed() {
  //switch actve particle
    kAct = (kAct+1) % Nparticles;
  //manual photon-nudging
    if (PhotNudgeToggle==true && AutoNudgeToggle==false) {
      if (key == RETURN || key == ENTER) {
        NudgeToggle = true;
      }
    }
  //rotate particle at will:
    if (keyCode == LEFT || keyCode == UP) {
        Angle = Angle - PI*10/180.0;
        if (BFieldToggle==true) {
          BAngle = BAngle - PI*10/180.0;
          BAngle = BAngle % (2*PI);
          controlP5.controller("BAngleSL").setValue(BAngle*180.0/PI);
        }
    }
    if (keyCode == RIGHT || keyCode == DOWN) {
        Angle = Angle + PI*10/180.0;
        if (BFieldToggle==true) {
          BAngle = BAngle + PI*10/180.0;
          BAngle = BAngle % (2*PI);
          controlP5.controller("BAngleSL").setValue(BAngle*180.0/PI);
        }
    }
}

void keyReleased() {
  if (PhotNudgeToggle==true && AutoNudgeToggle==false) {
    if (key == RETURN || key == ENTER) {
     NudgeToggle = false;
    }
  }
}
